-- Warborg is Wargames playing on network
-- Copyright (C) 2011 Manuel De Girardi

-- This Program is Free Software: You Can Redistribute It and/or Modify
-- It Under The Terms of The GNU General Public License As Published By
-- The Free Software Foundation, Either Version 3 of The License, or
-- (at Your Option) Any Later Version.

-- This Program is Distributed in The Hope That It Will Be Useful,
-- But WITHOUT ANY WARRANTY; Without Even The Implied Warranty of
-- MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See The
-- GNU General Public License for More Details.

-- You Should Have Received A Copy of The GNU General Public License
-- Along with This Program.  if not, See <Http://Www.Gnu.Org/Licenses/>.
with Warborg.Common;
with Warborg.World;
use Warborg.World;
with Warborg.Weapon.Manager;



with Ada.Text_Io;
use Ada;
with Ada.Float_Text_Io;
use Ada;

with Gtk.Main;
use Gtk;
with Gtk.Handlers;
with Gdk.Window;                        use Gdk.Window;
with Gtk.Widget;                        use Gtk.Widget;
with Pango.Layout;                      use Pango.Layout;
with Gtk.Style;                         use Gtk.Style;
with Gdk.Drawable;                      use Gdk.Drawable;
with Glib.Error;                        use Glib.Error;
with Interfaces.C;
use Interfaces.C;
with Ada.Numerics.Discrete_Random;
with Ada.Numerics.Float_Random;
use Ada.Numerics;

with Ada.Calendar.Formatting;
use Ada.Calendar;

package body Warborg.Xborg is
   
   use Warborg.World, Warborg.Common, Warborg.Common.My_Map;
   use Warborg;
   
   procedure Split(Une : Duration;
                   Heure : out Natural;
                   Minute : out Natural;
                   Seconde : out Natural;
                   Reste   : out Natural) is
      S : Duration := Une;
      Tab_I : array (Integer range 1..4) of duration :=
        (3600.0,60.0,1.0,0.0001);
      Tab_Res : array (Integer range 1..4) of Natural := (others => 0);

   begin
      Heure := 0;
      Minute := 0;
      Seconde := 0;
      Reste := 0;
      for I in 1..4 loop
         while S >= Tab_I(I) loop
            S := S - Tab_I(I);
            Tab_Res(I) := Tab_Res(I)+1;
         end loop;
      end loop;
      Heure := Tab_Res(1);
      Minute := Tab_Res(2);
      Seconde := Tab_Res(3);
      Reste := Tab_Res(4);
   end Split;
   
   
   task body Borg_Type is
      Launch_Code : String_Access;
      Alphabet   : String := "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
      position    : array (1..10) of Positive := (others => 1);
      Index, k : Positive := 1;
      Code : String(1..10) := (others => '0');
      End_Of_Task : Boolean := False;
   begin

      while not End_Of_Task Loop
         select
            accept Start(Code : in String) do
               position  := (others => 1);
               Launch_Code := new String ' (Code);
               K := Launch_Code'Length;
               Text_Io.New_Line;
            end Start;
         or
            accept Stop do
               End_Of_Task := True;
            end Stop;
         end select;
         while not End_Of_Task loop

            for J in Launch_Code'range loop
               Text_Io.Put(Alphabet(Position(J)));
               Code(J) := Alphabet(Position(J));
            end loop;
	    delay 0.0025;
            if Code(Launch_Code'Range) = Launch_Code.all then
               exit;
            else
               Text_Io.Put(Character'Val(13));
            end if;
            select
               accept Stop do
                  End_Of_Task := True;
               end Stop;
            else
               for I in Launch_Code'range loop
                  Position(I) := Position(I) + 1;
                  if Position(I) <= Alphabet'Length then
                     exit;
                  else
                     Position(I) := 1;
                  end if;
               end loop;
            end select;
         end loop;
      end loop;
   end Borg_Type;

   
   
   
   
   
   Green_Gc : Gdk.GC.Gdk_GC;
   Layout : Pango_Layout;
   
   procedure Draw_wopr (Item : access Display_Item_Record'Class) is
   begin
      Set_Foreground (Green_Gc, Display_Item (Item).Color);
      Draw_Rectangle
        (Pixmap (Item),
         GC     => Green_Gc,
         Filled => True,
         X      => 0,
         Y      => 0,
         Width  => Item.W,
         Height => Item.H);
      Set_Foreground (Green_Gc, Black (Get_Default_Colormap));

      Set_Text (Layout, "Wopr");
      Draw_Layout
        (Pixmap (Item),
         Green_Gc,
         10,
         10,
         Layout);

      Draw_Shadow
        (Style       => Get_Style (Item.Canvas),
         Window      => Pixmap (Item),
         State_Type  => State_Normal,
         Shadow_Type => Shadow_Out,
         X           => 0,
         Y           => 0,
         Width       => Item.W,
         Height      => Item.H);


   end Draw_wopr;



   ----------------
   -- Initialize --
   ----------------

   procedure Initialize_Wopr(Xborg : Xborg_Application;
                             Item   : access Display_Item_Record'Class) is
   begin
      Item.Canvas := Interactive_Canvas (Xborg.Canvas);
      Item.Color := Colors (2);
      Item.W := 10;
      Item.H := 10;
      Item.Num := 1;
      Xborg.Items_List (Item.Num) := Canvas_Item (Item);
      Xborg.Num := Xborg.Num + 1;
      Set_Screen_Size (Item, Item.W, Item.H);

      Draw_wopr (Item);
   end Initialize_wopr;

   ---------------------
   -- Add_Single_Item --
   ---------------------

   procedure Add_Wopr_Item (Xborg : Xborg_Application) is

      Item : constant Display_Item := new Display_Item_Record;
   begin
      Initialize_wopr (Xborg, Item);

      Put (Xborg.Canvas, Item,
           Gint(Map_X_Coordonate_Type'Truncation(Xborg.Xborg.Wopr.Wopr_Attributes.Source.X)),
           Gint(Map_Y_Coordonate_Type'Truncation(Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y)));

      Refresh_Canvas (Xborg.Canvas);
      Show_Item (Xborg.Canvas, Item);
   end Add_Wopr_Item;



   procedure Draw_Game_Over (Item : access Display_Item_Record'Class) is
   begin
      Set_Foreground (Green_Gc, Display_Item (Item).Color);
      Draw_Rectangle
        (Pixmap (Item),
         GC     => Green_Gc,
         Filled => True,
         X      => 0,
         Y      => 0,
         Width  => Item.W,
         Height => Item.H);
      Set_Foreground (Green_Gc, Black (Get_Default_Colormap));

      Set_Text (Layout, "Game Over");
      Draw_Layout
        (Pixmap (Item),
         Green_Gc,
         10,
         10,
         Layout);

      Draw_Shadow
        (Style       => Get_Style (Item.Canvas),
         Window      => Pixmap (Item),
         State_Type  => State_Normal,
         Shadow_Type => Shadow_Out,
         X           => 0,
         Y           => 0,
         Width       => Item.W,
         Height      => Item.H);


   end Draw_Game_over;



   ----------------
   -- Initialize --
   ----------------

   procedure Initialize_Game_over(Xborg : Xborg_Application;
                                  Item   : access Display_Item_Record'Class) is
   begin
      Item.Canvas := Interactive_Canvas (Xborg.Canvas);
      Item.Color := Colors (16);
      Item.W := 100;
      Item.H := 35;
      Item.Num := 1;

      Set_Screen_Size (Item, Item.W, Item.H);

      Draw_Game_over (Item);
   end Initialize_Game_over;

   ---------------------
   -- Add_Single_Item --
   ---------------------

   procedure Add_Game_Over_item (Xborg : Xborg_Application) is

      Item : constant Display_Item := new Display_Item_Record;
   begin
      Initialize_Game_over (Xborg, Item);
      
      Put (Xborg.Canvas, Item,
           Gint(width/2.0),
           Gint(heigh/2.0));

      Refresh_Canvas (Xborg.Canvas);
      Show_Item (Xborg.Canvas, Item);
   end Add_Game_Over_item;

   package Defcon_Timeout is
      new Gtk.Main.Timeout(Xborg_Application);



   package Main_Handlers is
      new Gtk.Handlers.User_Callback(Gtk.Widget.Gtk_Widget_Record, Xborg_Application);
   package Main_Return_Handlers is
      new Gtk.Handlers.User_Return_Callback(Gtk.Widget.Gtk_Widget_Record, Boolean, Xborg_Application);




   procedure Main_Quit
     (widget : access Gtk_Widget_Record'class;
      Xborg : Xborg_Application) is
      Dialog : Gtk.Dialog.Gtk_Dialog;
      Yes    : Gtk.Widget.Gtk_Widget;
      No     : Gtk.Widget.Gtk_Widget;
      Help   : Gtk.Widget.Gtk_Widget;
   begin
      
      
      
      loop
         Gtk.Dialog.Gtk_New(Dialog, "Quit ?", Xborg.Main_Window, Gtk.dialog.modal);
         Yes := Gtk.Dialog.Add_Button(Dialog, "Yes", Gtk.Dialog.Gtk_Response_Yes);
         No := Gtk.Dialog.Add_Button(Dialog, "No", Gtk.Dialog.Gtk_Response_No);
         Help := Gtk.Dialog.Add_Button(Dialog, "Help", Gtk.Dialog.Gtk_Response_Help);
	 
	 
	 
         case Gtk.Dialog.Run(Dialog) is
	    
	    
	    when Gtk.Dialog.Gtk_Response_Yes    =>
	       Command_type'Output(Xborg.Channel, Wopr_get);
	       Xborg.Success := Boolean'Input(Xborg.Channel);	       	    	    	    
               Gtk.Dialog.Destroy(Dialog);
               Destroy(Xborg.Main_Window);
               Gtk.Main.Main_quit;
               exit;
            when Gtk.Dialog.Gtk_Response_No     =>
               Gtk.Dialog.Destroy(Dialog);
               exit;
            when Gtk.Dialog.Gtk_Response_Help   =>
               Gtk.Dialog.Destroy(Dialog);
            when others =>
               Gtk.Dialog.Destroy(Dialog);
         end case;
	 
      end loop;
   end Main_Quit;


   function Main_Destroy
     (widget : access Gtk_Widget_Record'class;
      Xborg : Xborg_Application) return Boolean is
      Dialog : Gtk.Dialog.Gtk_Dialog;
      Yes    : Gtk.Widget.Gtk_Widget;
      No     : Gtk.Widget.Gtk_Widget;
      Help   : Gtk.Widget.Gtk_Widget;
   begin

      loop
         Gtk.Dialog.Gtk_New(Dialog, "Quit ?", Xborg.Main_Window, Gtk.dialog.modal);
         Yes := Gtk.Dialog.Add_Button(Dialog, "Yes", Gtk.Dialog.Gtk_Response_Yes);
         No := Gtk.Dialog.Add_Button(Dialog, "No", Gtk.Dialog.Gtk_Response_No);
         Help := Gtk.Dialog.Add_Button(Dialog, "Help", Gtk.Dialog.Gtk_Response_Help);
         case Gtk.Dialog.Run(Dialog) is
            when Gtk.Dialog.Gtk_Response_Yes    =>
	       Command_type'Output(Xborg.Channel, Wopr_get);
	       Xborg.Success := Boolean'Input(Xborg.Channel);	       
               Gtk.Dialog.Destroy(Dialog);
               Destroy(Xborg.Main_Window);
               Gtk.Main.Main_quit;
               return False;
            when Gtk.Dialog.Gtk_Response_No     =>
               Gtk.Dialog.Destroy(Dialog);
               return True;
            when Gtk.Dialog.Gtk_Response_Help   =>
               Gtk.Dialog.Destroy(Dialog);
            when others =>
               Gtk.Dialog.Destroy(Dialog);
         end case;
      end loop;

   end Main_Destroy;

   procedure Draw_Background
     (Canvas        : access Image_Canvas_Record;
      Screen_Rect   : Gdk.Rectangle.Gdk_Rectangle)
   is
      X_Left : constant Glib.Gint := Left_World_Coordinates (Canvas);
      Y_Top  : constant Glib.Gint := Top_World_Coordinates (Canvas);
   begin
      if Canvas.Background /= null then
         --  This is slightly complex, since we need to properly handle zooming
         --  and tiling.
         declare
            X, Y, W, H, Ys : Gint;
            Xs : Gint := Screen_Rect.X;
            Bw : constant Gint := Get_Width (Canvas.Background)
              * Gint (Get_Zoom (Canvas)) / 100;
            Bh : constant Gint := Get_Height (Canvas.Background)
              * Gint (Get_Zoom (Canvas)) / 100;
            Scaled : Gdk_Pixbuf := Canvas.Background;
         begin
            --  A real application would cache this scaled pixmap, and update
            --  the cache when the "zoomed" signal is emitted.
            if Get_Zoom (Canvas) /= 100 then
               Scaled := Scale_Simple (Canvas.Background, Bw, Bh);
            end if;

            while Xs < Screen_Rect.X + Screen_Rect.Width loop
               Ys := Screen_Rect.Y;
               X := (X_Left + Xs) mod Bw;
               W := Gint'Min (Screen_Rect.Width + Screen_Rect.X - Xs, Bw - X);

               while Ys < Screen_Rect.Y + Screen_Rect.Height loop
                  Y := (Y_Top  + Ys) mod Bh;
                  H := Gint'Min
                    (Screen_Rect.Height + Screen_Rect.Y - Ys, Bh - Y);

                  Render_To_Drawable
                    (Pixbuf       => Scaled,
                     Drawable     => Get_Window (Canvas),
                     Gc           => Get_Black_GC (Get_Style (Canvas)),
                     Src_X        => X,
                     Src_Y        => Y,
                     Dest_X       => Xs,
                     Dest_Y       => Ys,
                     Width        => W,
                     Height       => H);
                  Ys := Ys + H;
               end loop;
               Xs := Xs + W;
            end loop;

            if Get_Zoom (Canvas) /= 100 then
               Unref (Scaled);
            end if;
         end;

      else
         Draw_Rectangle
           (Get_Window (Canvas),
            Get_Background_GC (Get_Style (Canvas), State_Normal),
            Filled => True,
            X      => Screen_Rect.X,
            Y      => Screen_Rect.Y,
            Width  => Gint (Screen_Rect.Width),
            Height => Gint (Screen_Rect.Height));
      end if;
      if Canvas.Draw_Grid then
         Draw_Grid (Interactive_Canvas (Canvas), Canvas.Grid_GC, Screen_Rect);
      end if;
   end Draw_Background;


   procedure Set_Background(Widget : access Gtk_Widget_Record'Class;
                            Xborg  : Xborg_Application)
   is
      Error : GError;
   begin
      Gdk_New_From_File
        (Xborg.Canvas.Background,
         Filename => "data/images/world-map-1.1.jpg",
         Error    => Error);
      Xborg.Canvas.Grid_GC := Get_White_GC (Get_Style (Xborg.Canvas));
      Refresh_Canvas (Xborg.Canvas);
   end Set_Background;


   procedure Add_Canvas_Link
     (Canvas : access Interactive_Canvas_Record'Class;
      Item1, Item2 : access Canvas_Item_Record'Class; Text : String := "");
   --  Add a link between Item1 and Item2

   procedure Draw_Missile (Item : access Display_Item_Record'Class) is
   begin
      Set_Foreground (Green_Gc, Display_Item (Item).Color);
      Draw_Rectangle
        (Pixmap (Item),
         GC     => Green_Gc,
         Filled => True,
         X      => 0,
         Y      => 0,
         Width  => Item.W,
         Height => Item.H);
      Set_Foreground (Green_Gc, Black (Get_Default_Colormap));

      Set_Text (Layout, "Missile" & Positive'Image (Display_Item (Item).Num/2));
      Draw_Layout
        (Pixmap (Item),
         Green_Gc,
         10,
         10,
         Layout);

      Draw_Shadow
        (Style       => Get_Style (Item.Canvas),
         Window      => Pixmap (Item),
         State_Type  => State_Normal,
         Shadow_Type => Shadow_Out,
         X           => 0,
         Y           => 0,
         Width       => Item.W,
         Height      => Item.H);
   end Draw_Missile;

   procedure Draw_Population (Item : access Display_Item_Record'Class) is
   begin
      Set_Foreground (Green_Gc, Display_Item (Item).Color);
      Draw_Rectangle
        (Pixmap (Item),
         GC     => Green_Gc,
         Filled => True,
         X      => 0,
         Y      => 0,
         Width  => Item.W,
         Height => Item.H);
      Set_Foreground (Green_Gc, Black (Get_Default_Colormap));

      Set_Text (Layout, "Population");-- & Positive'Image (Display_Item (Item).Num/2));
      Draw_Layout
        (Pixmap (Item),
         Green_Gc,
         10,
         10,
         Layout);

      Draw_Shadow
        (Style       => Get_Style (Item.Canvas),
         Window      => Pixmap (Item),
         State_Type  => State_Normal,
         Shadow_Type => Shadow_Out,
         X           => 0,
         Y           => 0,
         Width       => Item.W,
         Height      => Item.H);
   end Draw_Population;



   procedure Draw_Target (Item : access Display_Item_Record'Class) is
   begin
      Set_Foreground (Green_Gc, Display_Item (Item).Color);
      Draw_Rectangle
        (Pixmap (Item),
         GC     => Green_Gc,
         Filled => True,
         X      => 0,
         Y      => 0,
         Width  => Item.W,
         Height => Item.H);
      Set_Foreground (Green_Gc, Black (Get_Default_Colormap));

      Set_Text (Layout, "Target" & Positive'Image (Display_Item (Item).Num/2 -1));
      Draw_Layout
        (Pixmap (Item),
         Green_Gc,
         10,
         10,
         Layout);

      Draw_Shadow
        (Style       => Get_Style (Item.Canvas),
         Window      => Pixmap (Item),
         State_Type  => State_Normal,
         Shadow_Type => Shadow_Out,
         X           => 0,
         Y           => 0,
         Width       => Item.W,
         Height      => Item.H);

   end Draw_Target;



   -- edit missiles properties.

   package Missile_Handlers is
      new Gtk.Handlers.User_Callback(Gtk_Widget_Record, Xborg_Application);
   procedure On_select (Item   : access Gtk_Widget_Record'class;
                        Xborg  : Xborg_Application) is

   begin
      Text_Io.Put("Edit properties not available");
   end On_Select;


   procedure Initialize_Missile(Xborg : Xborg_Application;
                                Item   : access Display_Item_Record'Class;
                               Source_X : Gint := 0;
                               Source_Y : Gint := 0) is
   begin
      Item.Canvas := Interactive_Canvas (Xborg.Canvas);
      Item.Color := Colors (1);
      Item.W := 5;
      Item.H := 5;
      Item.Num := Xborg.Num + 1;
      if Xborg.num <= Xborg.Items_List'Last then
         Xborg.Items_List (Item.Num) := Canvas_Item (Item);
      end if;
      Xborg.Num := Xborg.Num + 1;
      Set_Screen_Size (Item, Item.W, Item.H);
      Draw_missile (Item);

      --Missile_handlers.Connect
      --  (Gtk_Widget_Record'class(Xborg.Items_List (Item.Num))'access, "item_selected",  Missile_Handlers.To_Marshaller (On_select'Access), Xborg);

   end Initialize_Missile;

   
   
   procedure Initialize_Population(Xborg : Xborg_Application;
				   Item   : access Display_Item_Record'Class;
				   Source_X : Gint := 0;
				   Source_Y : Gint := 0) is
   begin
      Item.Canvas := Interactive_Canvas (Xborg.Canvas);
      Item.Color := Colors (16);
      Item.W := 5;
      Item.H := 5;
      Item.Num := Xborg.Num + 1;
      if Xborg.num <= Xborg.Items_List'Last then
         Xborg.Items_List (Item.Num) := Canvas_Item (Item);
      end if;
      Xborg.Num := Xborg.Num + 1;
      Set_Screen_Size (Item, Item.W, Item.H);
      Draw_missile (Item);

      --Missile_handlers.Connect
      --  (Gtk_Widget_Record'class(Xborg.Items_List (Item.Num))'access, "item_selected",  Missile_Handlers.To_Marshaller (On_select'Access), Xborg);

   end Initialize_Population;

   
   
   
   
   procedure Initialize_target(Xborg : Xborg_Application;
                               Item   : access Display_Item_Record'Class) is
   begin
      Item.Canvas := Interactive_Canvas (Xborg.Canvas);
      Item.Color := Colors (2);
      Item.W := 5;
      Item.H := 5;
      Item.Num := Xborg.Num + 1;
      if Xborg.num <= Xborg.Items_List'Last then
         Xborg.Items_List (Item.Num) := Canvas_Item (Item);
      end if;
      Xborg.Num := Xborg.Num + 1;
      Set_Screen_Size (Item, Item.W, Item.H);
      Draw_Target (Item);
   end Initialize_target;

   
   use Warborg.Weapon;
   ---------------------
   -- Add_Single_Item --
   ---------------------
   
   procedure Add_Missile_With_Cible(Xborg : in Xborg_Application;
                                   Source_X : Gint;
                                   Source_Y : Gint;
                                   Target_X : Gint;
                                   Target_Y : Gint;
                                   Charge   : Charge_Type) is
      Missile : constant Display_Item := new Display_Item_Record;
      cible : constant Display_Item := new Display_Item_Record;
   begin
      Initialize_Missile (Xborg, Missile);
      Initialize_target (Xborg, cible);
      --Add_Canvas_Link (Xborg.Canvas, Xborg.Items_List(1), Missile, "");
      Add_Canvas_Link (Xborg.Canvas, Missile, Cible, "");--, "Charge : " & Charge_Type'Image(Charge));

      Put (Xborg.Canvas, Missile, Source_X, Source_Y);
      Put (Xborg.Canvas, Cible, Target_X, Target_Y);
      Refresh_Canvas (Xborg.Canvas);
      Show_Item (Xborg.Canvas, Missile);
      Show_Item (Xborg.Canvas, Cible);
   end Add_Missile_With_cible;




   procedure Add_Canvas_Link
     (Canvas : access Interactive_Canvas_Record'Class;
      Item1, Item2 : access Canvas_Item_Record'Class; Text : String := "")
   is
      Link : constant Canvas_Link := new Canvas_Link_Record;
   begin
      Add_Link (Canvas, Link, Item1, Item2, Both_Arrow, Text);
   end Add_Canvas_Link;



   procedure Draw_External_Missil (Item : access Display_Item_Record'Class) is
   begin
      Set_Foreground (Green_Gc, Display_Item (Item).Color);
      Draw_Rectangle
        (Pixmap (Item),
         GC     => Green_Gc,
         Filled => True,
         X      => 0,
         Y      => 0,
         Width  => Item.W,
         Height => Item.H);
      Set_Foreground (Green_Gc, Black (Get_Default_Colormap));
      Set_Text (Layout, "Missile" & Positive'Image (Display_Item (Item).Num));
      Draw_Layout
        (Pixmap (Item),
         Green_Gc,
         0,
         0,
         Layout);

      Draw_Shadow
        (Style       => Get_Style (Item.Canvas),
         Window      => Pixmap (Item),
         State_Type  => State_Normal,
         Shadow_Type => Shadow_Out,
         X           => 0,
         Y           => 0,
         Width       => Item.W,
         Height      => Item.H);
   end Draw_External_Missil;


   procedure Initialize_External_Missil(Xborg : Xborg_Application;
                                        Item   : access Display_Item_Record'Class) is
   begin
      Item.Canvas := Interactive_Canvas (Xborg.Canvas);
      Item.Color := Colors (3);
      Item.W := 8;
      Item.H := 8;
      Item.Num := Xborg.External_Num;
      if Xborg.External_Num <= Xborg.External_Items_List'Last then
         Xborg.External_Items_List (Item.Num) := Canvas_Item (Item);
         Xborg.External_Num := Xborg.External_Num + 1;
      end if;
      Set_Screen_Size (Item, Item.W, Item.H);
      Draw_External_Missil (Item);
   end Initialize_External_Missil;



   
   procedure Add_Missile(Xborg : in Xborg_Application;
                        Source_X : in Gint;
                        Source_Y : in Gint) is

      missil : constant Display_Item := new Display_Item_Record;

   begin
      Initialize_External_Missil (Xborg, missil);
      Add_Canvas_Link (Xborg.Canvas, missil, missil, "");
      Put (Xborg.Canvas, Missil, Source_X, Source_Y);
      Refresh_Canvas (Xborg.Canvas);
      Show_Item (Xborg.Canvas, Missil);
   end Add_Missile;
   
   
   
   procedure Add_Population(Xborg : in Xborg_Application;
			    Source_X : in Gint;
			    Source_Y : in Gint) is
      
      Population : constant Display_Item := new Display_Item_Record;

   begin
      Initialize_Population (Xborg, Population);
      --Add_Canvas_Link (Xborg.Canvas, Population, Population, "");
      Put (Xborg.Canvas, Population, Source_X, Source_Y);
      Refresh_Canvas (Xborg.Canvas);
      Show_Item (Xborg.Canvas, Population);
   end Add_population;
   
   
   procedure Print (GPS_Coordonates : GPS_Coordonates_Type) is
      Longitude : Longitude_Coordonate_Type := GPS_Coordonates.Longitude;
      Latitude  : Latitude_Coordonate_Type := GPS_Coordonates.Latitude;
   begin
      Text_Io.Put("Long ");
      Float_Text_Io.Put(Longitude.Degr_Lon, 3, 0, 0);Text_Io.Put("°");
      Float_Text_Io.Put(Longitude.Minu_Lon, 2, 0, 0);Text_Io.Put(''');
      Float_Text_Io.Put(Longitude.Seco_Lon, 2, 0, 0);Text_Io.Put('"');
      Text_Io.Put(", Lat ");
      Float_Text_Io.Put(Latitude.Degr_Lat, 3, 0, 0);Text_Io.Put("°");
      Float_Text_Io.Put(Latitude.Minu_Lat, 2, 0, 0);Text_Io.Put(''');
      Float_Text_Io.Put(Latitude.Seco_Lat, 2, 0, 0);Text_Io.Put('"');
      Text_Io.New_Line;
   end Print;
   
   
   procedure Print_Missile(Missile : Missile_Type) is
   begin
      Print(Missile.Missile_Attributes.Position);
   end Print_Missile;
   

   
   procedure Draw_Wopr_With_Missiles(Xborg : in Xborg_Application) is
      
      
      
      
      
   begin
      Xborg.Items_List := (others => null);

      Xborg.Num := 1;

      Add_Wopr_Item(Xborg);
      
      if not Weapon.Manager.Is_Empty(Xborg.Xborg.Wopr.Weapons) then
         for I in 1..Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons) loop
            declare
               Missile : Missile_type := Weapon.Manager.Element(Xborg.Xborg.Wopr.Weapons, I);
	       Source_Map_Coordonates : Map_Coordonates_Type := To_Map_Coordonates(Missile.Missile_Attributes.Position);
	       Target_Map_Coordonates : Map_Coordonates_Type := To_Map_Coordonates(Missile.Missile_Attributes.target);
            begin
	       Print_Missile(Missile);
	       
               Add_Missile_With_Cible(Xborg,
				      Gint(Source_Map_Coordonates.X),
				      Gint(Source_Map_Coordonates.Y),
				      Gint(Target_Map_Coordonates.X),
				      Gint(Target_Map_Coordonates.Y),
				      Missile.Missile_Attributes.charge);

            end;
         end loop;
      end if;
      
      

      Xborg.Xborg.Ouest_Population := 0;
      Xborg.Xborg.Est_Population := 0;
      if not Nation.Manager.Is_Empty(Xborg.Xborg.Wopr.Populations) then
         for I in 1..Nation.Manager.Last_Index(Xborg.Xborg.Wopr.populations) loop
            declare
               Nation : Nation_type := Warborg.Nation.Manager.Element(Xborg.Xborg.Wopr.populations, I);
	       Source_Map_Coordonates : Map_Coordonates_Type := To_Map_Coordonates(Nation.Gps_coordonates);	       
            begin
	       --Print_Missile(Missile);
	       case Side_Of(Nation.Gps_Coordonates) is
		  when Ouest =>
		     Xborg.Xborg.Ouest_Population := Xborg.Xborg.Ouest_Population + Nation.population;
		  when Est =>
		     Xborg.Xborg.Est_Population := Xborg.Xborg.Est_Population + Nation.population;
	       end case;
               Add_Population(Xborg,
			      Gint(Source_Map_Coordonates.X),
			      Gint(Source_Map_Coordonates.Y));
            end;
         end loop;
      end if;
      
   end Draw_Wopr_With_Missiles;


   procedure Clear_canvas (widget : access Gtk_Widget_Record'class;
                           Xborg : in Xborg_Application) is
      function Remove_Internal
        (Canvas : access Interactive_Canvas_Record'Class;
         Item   : access Canvas_Item_Record'Class) return Boolean is

      begin

         Remove (Canvas, Item);
         return True;
      end Remove_Internal;

   begin

      For_Each_Item (Xborg.Canvas, Remove_Internal'Unrestricted_Access);

      Refresh_Canvas (Xborg.Canvas);
   end Clear_canvas;




   package Code_Random is
      function Random(Length : in Positive) return String;
      procedure  Reset;
   end Code_Random;

   package body Code_Random is

      Alphabet : String := "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

      subtype Digit_Type is Positive range 1..Alphabet'Length;

      package Digit_Random is new Discrete_Random(Digit_Type);
      use Digit_Random;
      Digit_Gen : Generator;

      procedure  Reset is
      begin
         Reset(Digit_Gen);
      end Reset;

      function Random(Length : in Positive) return String is

         Code : String(1..Length);
      begin
         for I in Code'Range loop
            Code(I) := Alphabet(Random(Digit_Gen));
         end loop;
         return Code;
      end Random;
   end Code_Random;

   use Code_Random;

   use Float_Random;

   Position_Gen : Generator;

   function Defcon_Automation(Xborg : Xborg_Application) return Boolean;

   procedure Get_Wopr_Attributes(widget : access Gtk_Widget_Record'class;
                                 Xborg : Xborg_Application) is
      Vbox      : Gtk_Vbox;
      Ok        : Gtk.Widget.Gtk_Widget;
      cancel    : Gtk.Widget.Gtk_Widget;
      Degr_Lat : Gdouble := 0.0;
      Minu_Lat : Gdouble := 0.0;
      Seco_Lat : Gdouble := 0.0;
      Degr_Lon : Gdouble := 0.0;
      Minu_Lon : Gdouble := 0.0;
      Seco_Lon : Gdouble := 0.0;

      Lon_Unit : constant Float := width/(360.0*60.0*60.0);
      Lat_Unit : constant Float := heigh/(180.0*60.0*60.0);

      Missile_Available : Boolean := False;
      Missiles_Count : Missile_Vector_Index_Type;
      Map_Coordonates : Map_Coordonates_Type;
      Gps_Coordonates : Gps_Coordonates_Type;
      
      Poupulation_Available : Boolean := False;
      Poupulations_Count    : natural := 0;
      
      
      
   begin

      Clear_canvas(null, Xborg);
      Command_type'Output(Xborg.Channel, Wopr_get);
      Xborg.Success := Boolean'Input(Xborg.Channel);
      Reset;
      Reset(Position_Gen);
      if Xborg.Success then
         Xborg.Xborg.Wopr.Defcon := Defcon_5;
         Xborg.Xborg.Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Xborg.Channel);
         Weapon.Manager.Clear(Xborg.Xborg.Wopr.Weapons);	 
         Missile_Available := Boolean'Input(Xborg.Channel);
         if Missile_Available then
            Missiles_Count := Missile_Vector_Index_Type'Input(Xborg.Channel);
            for I in 1..Missiles_Count loop
               Xborg.Missile_Attributes := Missile_Attributes_Type'Input(Xborg.Channel);
               Xborg.One_Missile.Missile_Attributes := Xborg.Missile_Attributes;
               Weapon.Manager.Insert(Xborg.Xborg.Wopr.weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons)+ 1, Xborg.One_Missile, 1);
            end loop;
         end if;
         --Set_Sensitive(Xborg.Create_one_Item, True);
	 
	 Nation.Manager.Clear(Xborg.Xborg.Wopr.populations);
	 Poupulation_Available := Boolean'Input(Xborg.Channel);
         if Poupulation_Available then
            Poupulations_Count := Natural(positive'Input(Xborg.Channel));
            for I in 1..Poupulations_Count loop
               Nation.Manager.Insert(Xborg.Xborg.Wopr.populations, Nation.Manager.Last_Index(Xborg.Xborg.Wopr.Populations)+ 1, nation_Type'Input(Xborg.Channel), 1);
            end loop;
         end if;
         
	 
	 
	 
         delay 0.1;

         Xborg.Num := 1;
         Xborg.Items_List := (others => null);
	 

         Draw_Wopr_With_Missiles(Xborg);


         --  if Is_Empty(Xborg.Xborg.Wopr.Missiles) then
         --     Set_Sensitive(Xborg.Launch_one_Item, False);
         --     Set_Sensitive(Xborg.Launch_All_Item, False);
         --  elsif not Is_Empty(Xborg.Xborg.Wopr.Missiles) then
         --     Set_Sensitive(Xborg.Launch_one_Item, True);
         --     Set_Sensitive(Xborg.Launch_All_Item, True);
         --  end if;


         Xborg.Timeout_Id := Defcon_Timeout.Add(10000, Defcon_Automation'Access, Xborg);
	 
      else
	 

         
         Gtk_New(Xborg.Wopr_Window.Dialog,
                 "Get Wopr coordonates",
                 Xborg.Main_Window,
                 Gtk.dialog.modal);
         Ok := Gtk.Dialog.Add_Button(Xborg.Wopr_Window.Dialog, "Ok", Gtk.Dialog.Gtk_Response_Ok);
         Cancel := Gtk.Dialog.Add_Button(Xborg.Wopr_Window.Dialog, "Cancel", Gtk.Dialog.Gtk_Response_Cancel);
         Vbox := Get_Vbox(Xborg.Wopr_Window.Dialog);

         Set_Position(Xborg.Wopr_Window.dialog, Win_Pos_Center);
	 Text_Io.Put_Line("Xborg wopr side := " & Side_Type'Image(Xborg.Xborg.Wopr.Side));
	 Xborg.Xborg.Wopr.Wopr_Attributes.side := Xborg.Xborg.Wopr.Side;
	 case Xborg.Xborg.Wopr.Side is
	    when Ouest =>
	       
	       Gtk_New(Xborg.Wopr_Window.Degr_Lat, -90.0, 90.0, 1.0);
	       Gtk_New(Xborg.Wopr_Window.Degr_Lon, -180.0, 0.0, 1.0);
	    when Est =>
	       Gtk_New(Xborg.Wopr_Window.Degr_Lat, -90.0, 90.0, 1.0);
	       Gtk_New(Xborg.Wopr_Window.Degr_Lon, 0.0, 180.0, 1.0);
	 end case;
         Gtk_New(Xborg.Wopr_Window.Minu_Lat, 0.0, 60.0, 1.0);
         Gtk_New(Xborg.Wopr_Window.Seco_Lat, 0.0, 60.0, 1.0);
         
         Gtk_New(Xborg.Wopr_Window.Minu_Lon, 0.0, 60.0, 1.0);
         Gtk_New(Xborg.Wopr_Window.Seco_Lon, 0.0, 60.0, 1.0);
         Gtk_New_Hbox(Xborg.Wopr_Window.Lat_Hbox);
         Gtk_New_Hbox(Xborg.Wopr_Window.Lon_Hbox);
         Gtk_New(Xborg.Wopr_Window.Lat_Label, "Latitude  :");
         Gtk_New(Xborg.Wopr_Window.Lon_Label, "Longitude :");
         Gtk_New_Hbox(Xborg.Wopr_Window.Lat_Hbox);
         Gtk_New_Hbox(Xborg.Wopr_Window.Lon_Hbox);
         Pack_Start(Xborg.Wopr_Window.Lat_Hbox, Xborg.Wopr_Window.Lat_Label);
         Pack_Start(Xborg.Wopr_Window.Lat_Hbox, Xborg.Wopr_Window.Degr_lat);
         Pack_Start(Xborg.Wopr_Window.Lat_Hbox, Xborg.Wopr_Window.Minu_lat);
         Pack_Start(Xborg.Wopr_Window.Lat_Hbox, Xborg.Wopr_Window.Seco_lat);
         Pack_Start(Xborg.Wopr_Window.Lon_Hbox, Xborg.Wopr_Window.Lon_Label);
         Pack_Start(Xborg.Wopr_Window.Lon_Hbox, Xborg.Wopr_Window.Degr_Lon);
         Pack_Start(Xborg.Wopr_Window.Lon_Hbox, Xborg.Wopr_Window.Minu_Lon);
         Pack_Start(Xborg.Wopr_Window.Lon_Hbox, Xborg.Wopr_Window.Seco_Lon);
         Pack_Start(Vbox, Xborg.Wopr_Window.Lat_Hbox);
         Pack_Start(Vbox, Xborg.Wopr_Window.Lon_Hbox);


         Show_All(vbox);

         case Gtk.Dialog.Run(Xborg.Wopr_Window.Dialog) is
            when Gtk.Dialog.Gtk_Response_Ok    =>





               Degr_Lat := Get_Value(Xborg.Wopr_Window.Degr_Lat);
               Minu_Lat := Get_Value(Xborg.Wopr_Window.Minu_Lat);
               Seco_Lat := Get_Value(Xborg.Wopr_Window.Seco_Lat);
               Degr_Lon := Get_Value(Xborg.Wopr_Window.Degr_Lon);
               Minu_Lon := Get_Value(Xborg.Wopr_Window.Minu_Lon);
               Seco_Lon := Get_Value(Xborg.Wopr_Window.Seco_Lon);
	       
	       Gps_Coordonates.Longitude := (Float(Degr_Lon), Float(Minu_Lon), Float(Seco_Lon));
	       Gps_Coordonates.Latitude := (Float(Degr_Lat), Float(Minu_Lat), Float(Seco_Lat));
	       Xborg.Xborg.Wopr.Wopr_Attributes.Source := To_Map_Coordonates(Gps_Coordonates);
	       
               --  if Degr_Lon >= 0.0 then
		  
		  
               --     Xborg.Xborg.Wopr.Wopr_Attributes.Source.X := Max_X_Coordonate/2.0 + Float(Degr_Lon) * (3600.0*Lon_unit) + Float(Minu_Lon) * (60.0* Lon_Unit) +
               --       Float(Seco_Lon) * Lon_Unit;

               --     if Xborg.Xborg.Wopr.Wopr_Attributes.Source.X > Max_X_Coordonate then
               --        Xborg.Xborg.Wopr.Wopr_Attributes.Source.X := Max_X_Coordonate - 1.0;
               --     end if;

               --  else
		  
		  
               --     Xborg.Xborg.Wopr.Wopr_Attributes.Source.X := Max_X_Coordonate/2.0 - ((Float(Degr_Lon)*(-1.0)) * (3600.0*Lon_unit) + Float(Minu_Lon) * (60.0* Lon_Unit) +
               --                                                             Float(Seco_Lon) * Lon_Unit);
               --     if Xborg.Xborg.Wopr.Wopr_Attributes.Source.X < 0.0 then
               --        Xborg.Xborg.Wopr.Wopr_Attributes.Source.X := 1.0;
               --     end if;
               --  end if;
	       
               --  if Degr_Lat >= 0.0 then


               --     Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y := Max_Y_Coordonate/2.0 + Float(Degr_Lat)* (3600.0*Lat_unit) + Float(Minu_Lat) * (60.0* Lat_Unit) +
               --       Float(Seco_Lat) * Lat_Unit;
               --     if Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y > Max_Y_Coordonate/2.0 then
               --        Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y := Max_Y_Coordonate/2.0-1.0;
               --     end if;
               --  else


               --     Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y := Max_Y_Coordonate/2.0 - ((Float(Degr_Lat)*(-1.0))* (3600.0*Lat_unit) + Float(Minu_Lat) * (60.0* Lat_Unit) +  Float(Seco_Lat) * Lat_Unit);
               --     if Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y < 0.0 then
               --        Xborg.Xborg.Wopr.Wopr_Attributes.Source.Y := 1.0;
               --     end if;

               --  end if;
	       
               Xborg.Xborg.Wopr.Wopr_Attributes.Charge := Xborg.Xborg.Wopr.Wopr_Attributes.Charge + 1.0;

               Command_type'Output(Xborg.Channel, Wopr_Create);
	       Xborg.Xborg.Wopr.Wopr_Attributes.Start_Time := Clock;
               Wopr_Attributes_Type'Output(Xborg.Channel, Xborg.Xborg.Wopr.Wopr_Attributes);
               Text_Io.Put_Line("Wopr_Create");
               Xborg.Success := Boolean'Input(Xborg.Channel);
	       
               Xborg.Xborg.Wopr.Defcon := Defcon_5;

               Command_type'Output(Xborg.Channel, Wopr_get);
               Xborg.Success := Boolean'Input(Xborg.Channel);

               if Xborg.Success then
                  Xborg.Xborg.Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Xborg.Channel);
		  
                  Weapon.Manager.Clear(Xborg.Xborg.Wopr.Weapons);
                  --Add_Wopr_Item(Xborg);
                  Missile_Available := Boolean'Input(Xborg.Channel);
                  if Missile_Available then
                     Missiles_Count := Missile_Vector_Index_Type'Input(Xborg.Channel);
                     for I in 1..Missiles_Count loop
                        Xborg.Missile_Attributes := Missile_Attributes_Type'Input(Xborg.Channel);
                        Xborg.One_Missile.Missile_Attributes := Xborg.Missile_Attributes;
                        Weapon.Manager.Insert(Xborg.Xborg.Wopr.Weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons)+ 1, Xborg.One_Missile, 1);
                     end loop;
                  end if;
                  --Set_Sensitive(Xborg.Create_one_Item, True);
		  
		  
	     
		  Nation.Manager.Clear(Xborg.Xborg.Wopr.populations);
		  Poupulation_Available := Boolean'Input(Xborg.Channel);
		  if Poupulation_Available then
		     Poupulations_Count := Natural(positive'Input(Xborg.Channel));
		     for I in 1..Poupulations_Count loop
			Nation.Manager.Insert(Xborg.Xborg.Wopr.populations, Nation.Manager.Last_Index(Xborg.Xborg.Wopr.Populations)+ 1, nation_Type'Input(Xborg.Channel), 1);
		     end loop;
		  end if;
     
		  
                  delay 0.1;

                  Xborg.Num := 1;
                  Xborg.Items_List := (others => null);


                  Draw_Wopr_With_Missiles(Xborg);

                  --  if Is_Empty(Xborg.Xborg.Wopr.Missiles) then
                  --     Set_Sensitive(Xborg.Launch_one_Item, False);
                  --     Set_Sensitive(Xborg.Launch_All_Item, False);
                  --  elsif not Is_Empty(Xborg.Xborg.Wopr.Missiles) then
                  --     Set_Sensitive(Xborg.Launch_one_Item, True);
                  --     Set_Sensitive(Xborg.Launch_All_Item, True);
                  --  end if;
                  Set_Sensitive(Xborg.New_Menu_Item, False);
                  --Set_Sensitive(Xborg.Create_one_Item, True);
                  Xborg.Timeout_Id := Defcon_Timeout.Add(10000, Defcon_Automation'Access, Xborg);
               end if;
               Gtk.Dialog.Destroy(Xborg.Wopr_Window.Dialog);
            when Gtk.Dialog.Gtk_Response_Cancel =>
	       Set_Sensitive(Xborg.New_Menu_Item, True);
               Gtk.Dialog.Destroy(Xborg.Wopr_Window.Dialog);
            when others =>
               Gtk.Dialog.Destroy(Xborg.Wopr_Window.Dialog);
         end case;
      end if;
   end Get_Wopr_Attributes;

   procedure Game_Over
     (widget : access Gtk_Widget_Record'class;
      Xborg : Xborg_Application) is

      Ok_dialog : Gtk_dialog;
      Ok        : Gtk_Widget;

   begin
      Text_Io.Put(Character'Val(7));
      delay 0.2;
      Text_Io.Put(Character'Val(7));
      delay 0.2;
      Text_Io.Put(Character'Val(7));
      delay 0.2;
      Text_Io.Put(Character'Val(7));
      Gtk.Main.Timeout_Remove(Xborg.Timeout_Id);
      Set_Sensitive(Xborg.New_Menu_Item, True);
      --  Set_Sensitive(Xborg.Launch_one_Item, False);
      --  Set_Sensitive(Xborg.Launch_All_Item, False);
      --  Set_Sensitive(Xborg.Create_one_Item, False);      
      Gtk.Dialog.Gtk_New(Ok_Dialog, "Game over", Xborg.Main_window, Gtk.dialog.modal);
      
      Ok := Gtk.Dialog.Add_Button(Ok_Dialog, "OK", Gtk.Dialog.Gtk_Response_Ok);

      Set_Default_Size(Ok_Dialog, 150, 50);
      case Gtk.Dialog.Run(Ok_Dialog) is
         when Gtk.Dialog.Gtk_Response_Ok =>
            Gtk.Dialog.Destroy(Ok_Dialog);
         when others =>
            null;
      end case;
   end Game_Over;



   procedure Launch_all(widget : access Gtk_Widget_Record'Class;
                        Xborg : in Xborg_Application) is
      Success : Boolean := False;
      Missiles_Count : Missile_Vector_Index_Type;
      
      Poupulation_Available : Boolean := False;
      Poupulations_Count    : natural := 0;
		  
         
   begin
      Command_Type'Output(Xborg.Channel, Launch_All);
      Xborg.Success := Boolean'Input(Xborg.Channel);
      
      Command_type'Output(Xborg.Channel, Wopr_get);
      Xborg.Success := Boolean'Input(Xborg.Channel);

      if Xborg.Success then
         Xborg.Xborg.Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Xborg.Channel);
         Weapon.Manager.Clear(Xborg.Xborg.Wopr.Weapons);
         Add_Wopr_Item(Xborg);
         if Boolean'Input(Xborg.Channel) then
            Missiles_Count := Missile_Vector_Index_Type'Input(Xborg.Channel);
            for I in 1..Missiles_Count loop
               Xborg.Missile_Attributes := Missile_Attributes_Type'Input(Xborg.Channel);
               Xborg.One_Missile.Missile_Attributes := Xborg.Missile_Attributes;
               Weapon.Manager.Insert(Xborg.Xborg.Wopr.Weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons)+ 1, Xborg.One_Missile, 1);
         end loop;
            null;
         end if;
         --Set_Sensitive(Xborg.Create_one_Item, True);
	 Nation.Manager.Clear(Xborg.Xborg.Wopr.populations);
	 Poupulation_Available := Boolean'Input(Xborg.Channel);
         if Poupulation_Available then
            Poupulations_Count := Natural(positive'Input(Xborg.Channel));
            for I in 1..Poupulations_Count loop
               Nation.Manager.Insert(Xborg.Xborg.Wopr.populations, Nation.Manager.Last_Index(Xborg.Xborg.Wopr.Populations)+ 1, nation_Type'Input(Xborg.Channel), 1);
            end loop;
         end if;
         
	 
	 Clear_canvas(null, Xborg);
         delay 0.1;


         Xborg.Num := 1;
         Xborg.Items_List := (others => null);

         if Xborg.Success then
	    
            Draw_Wopr_With_Missiles(Xborg);
	 else
	    --Add_Game_Over_Item(Xborg);	 
	    Game_Over(null, Xborg);
         end if;
         --  if Is_Empty(Xborg.Xborg.Wopr.Missiles) then
         --     Set_Sensitive(Xborg.Launch_one_Item, False);
         --     Set_Sensitive(Xborg.Launch_All_Item, False);
         --  elsif not Is_Empty(Xborg.Xborg.Wopr.Missiles) then
         --     Set_Sensitive(Xborg.Launch_one_Item, True);
         --     Set_Sensitive(Xborg.Launch_All_Item, True);
         --  end if;

      else
         
         Game_Over(null, Xborg);
      end if;
   end Launch_All;


   --  procedure get_One(widget : access Gtk_Widget_Record'class;
   --                    Xborg : in Xborg_Application) is


   --     Success : Boolean := False;
   --     Ok        : Gtk.Widget.Gtk_Widget;
   --     cancel    : Gtk.Widget.Gtk_Widget;
   --     Vbox : Gtk_Vbox;
   --  begin


   --     Gtk_New(Xborg.Launch_Window, "Get Missile Id", Xborg.Main_Window, Gtk.dialog.modal);


   --     Ok := Gtk.Dialog.Add_Button(Xborg.Launch_Window, "Ok", Gtk.Dialog.Gtk_Response_Ok);
   --     Cancel := Gtk.Dialog.Add_Button(Xborg.Launch_Window, "Cancel", Gtk.Dialog.Gtk_Response_Cancel);
   --     Vbox := Get_Vbox(Xborg.Launch_Window);
   --     Set_Position(Xborg.Launch_Window, Win_Pos_Center);

   --     Gtk_New(Xborg.One_Missile_Launch, 1.0, Gdouble(Missiles_Vectors.Last_Index(Xborg.Xborg.Wopr.Missiles)), 1.0);
   --     Pack_Start(Vbox, Xborg.One_Missile_Launch);

   --     Show_All(Xborg.Launch_Window);
   --     case Gtk.Dialog.Run(Xborg.Launch_Window) is
   --        when Gtk.Dialog.Gtk_Response_Ok =>

   --           Command_Type'Output(Xborg.Channel, Launch_One);
   --           Missile_Vector_Index_type'Output(Xborg.Channel, Missile_Vector_Index_Type(Get_Value(Xborg.One_Missile_Launch)));
   --           Xborg.Success := Boolean'Input(Xborg.Channel);
   --           Command_type'Output(Xborg.Channel, Wopr_get);
   --           Xborg.Success := Boolean'Input(Xborg.Channel);

   --           if Xborg.Success then
   --              Xborg.Xborg.Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Xborg.Channel);
   --              Clear(Xborg.Xborg.Wopr.Missiles);
   --              Add_Wopr_Item(Xborg);
   --              if Boolean'Input(Xborg.Channel) then
   --                 Missiles_Count := Missile_Vector_Index_Type'Input(Xborg.Channel);
   --                 for I in 1..Missiles_Count loop
   --                    Xborg.Missile_Attributes := Missile_Attributes_Type'Input(Xborg.Channel);
   --                    Xborg.One_Missile.Missile_Attributes := Xborg.Missile_Attributes;
   --                    Weapon.Manager.Insert(Xborg.Xborg.Wopr.missiles, Last_Index(Xborg.Xborg.Wopr.Missiles)+ 1, Xborg.One_Missile, 1);
   --                 end loop;
   --                 null;
   --              end if;
   --              Set_Sensitive(Xborg.Create_one_Item, True);


   --              Clear_canvas(null, Xborg.all'access);
   --              delay 0.1;


   --              Xborg.Num := 1;
   --              Xborg.Items_List := (others => null);

   --              if Xborg.Success then

   --                 Draw_Wopr_With_Missiles(Xborg.all'access);

   --              end if;
   --              if Is_Empty(Xborg.Xborg.Wopr.Missiles) then
   --                 Set_Sensitive(Xborg.Launch_one_Item, False);
   --                 Set_Sensitive(Xborg.Launch_All_Item, False);
   --              elsif not Is_Empty(Xborg.Xborg.Wopr.Missiles) then
   --                 Set_Sensitive(Xborg.Launch_one_Item, True);
   --                 Set_Sensitive(Xborg.Launch_All_Item, True);
   --              end if;

   --           else

   --              Add_Game_Over_Item(Xborg);
   --              Game_Over(null, Xborg);
   --           end if;



   --           Gtk.Dialog.Destroy(Xborg.Launch_Window);
   --        when Gtk.Dialog.Gtk_Response_Cancel =>
   --           Gtk.Dialog.Destroy(Xborg.Launch_Window);
   --        when others =>
   --           null;
   --     end case;
   --  end Get_One;



   -----------
   -- Clear --
   -----------
   
   --  procedure Clear_canvas (widget : access Gtk_Widget_Record'class;
   --                          Xborg : Xborg_Application) is
   --     function Remove_Internal
   --       (Canvas : access Interactive_Canvas_Record'Class;
   --        Item   : access Canvas_Item_Record'Class) return Boolean is

   --     begin

   --        Remove (Canvas, Item);
   --        return True;
   --     end Remove_Internal;

   --  begin

   --     For_Each_Item (Xborg.Canvas, Remove_Internal'Unrestricted_Access);

   --     Refresh_Canvas (Xborg.Canvas);
   --  end Clear_canvas;

   function UpDate(Xborg : Xborg_Application) return Boolean is

      Hour, Min, Sec, Left : Natural := 0;

   begin
      Split(Clock - Xborg.Xborg.Wopr.Wopr_Attributes.Start_time, Hour, Min, Sec, Left);

      Set_Label(Xborg.Elapsed_Time_Label,  "Elapsed time : "
                  & natural'Image(hour) &
                  natural'Image(min) &
                  natural'Image(Sec));
      Queue_draw(Xborg.Elapsed_Time_Label);
      return True;
   end UpDate;

   
   function observation(Xborg : Xborg_Application) return Boolean is
      
      Hour, Min, Sec, Left : Natural := 0;

   begin      
      if Xborg.End_Time_Of_Observation < Clock then	 
	 Set_Label(Xborg.Destroyed_Missiles_Per_Houre,  "Frequency : " & Float'Image(Float(Xborg.Xborg.Wopr.Wopr_Attributes.Destroyed_Missiles) - Xborg.Total_Destroyed_Per_Houre/3600.0));
	 Xborg.Total_Destroyed_Per_Houre := Float(Xborg.Xborg.Wopr.Wopr_Attributes.Destroyed_Missiles);
	 Xborg.End_Time_Of_Observation := Clock + 3600.0;
      end if;
      Xborg.Elapsed_Houre := Xborg.End_Time_Of_Observation - Clock;
      Set_Label(Xborg.Elapsed_Houre_Label,  "Freq Elapsed : " & Formatting.Image(Xborg.Elapsed_Houre));
      
      Queue_draw(Xborg.Elapsed_Houre_Label);
      Queue_draw(Xborg.Destroyed_Missiles_Per_Houre);
      return True;
   end Observation;

   
   
   function Tic_Tac_Toe return C.Int;
   pragma Import (C, Tic_Tac_Toe, "tictactoe");
   Tic_Tac_Toe_Switch : C.Int;

   
   package Bool_Random is new Ada.Numerics.Discrete_Random(Boolean);   
      

   function Defcon_Automation(Xborg : Xborg_Application) return Boolean is

      Missile_Available : Boolean := False;
      Enemy_Available   : Boolean := False;
      Defensive_Charge  : Charge_Type := 0.0;
      Offensive_Charge  : Charge_Type := 0.0;
      Charge_Per_Missile : Charge_Type := 0.0;
      Missiles_Count : Missile_Vector_Index_Type;
      Enemy_Count : Natural := 0;
      Bool_Gen : Bool_Random.Generator;
      
            Poupulation_Available : Boolean := False;
      Poupulations_Count    : natural := 0;

   begin
      --Bool_Random.Reset(Bool_Gen);
      --Text_Io.Put_Line("pass 1");
      Command_type'Output(Xborg.Channel, Wopr_get);
      Xborg.Success := Boolean'Input(Xborg.Channel);
      if not Xborg.Success then
         --Add_Game_Over_Item(Xborg);	 
         Game_Over(null, Xborg);
         return True;	 
      end if;
      

      
      if Xborg.Success then
         --Text_Io.Put_Line("pass 2");
         Set_label(Xborg.Earned_missiles_Label, "Earned Missiles : " & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Earned_Missiles));
         Xborg.Xborg.Wopr.Wopr_Attributes := Wopr_Attributes_Type'Input(Xborg.Channel);
         Weapon.Manager.Clear(Xborg.Xborg.Wopr.Weapons);

         Missile_Available := Boolean'Input(Xborg.Channel);
         if Missile_Available then
            Missiles_Count := Missile_Vector_Index_Type'Input(Xborg.Channel);
            for I in 1..Missiles_Count loop
               Xborg.Missile_Attributes := Missile_Attributes_Type'Input(Xborg.Channel);
               Xborg.One_Missile.Missile_Attributes := Xborg.Missile_Attributes;
               Weapon.Manager.Insert(Xborg.Xborg.Wopr.Weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons)+ 1, Xborg.One_Missile, 1);
            end loop;
         end if;
         --Set_Sensitive(Xborg.Create_one_Item, True);
	 Nation.Manager.Clear(Xborg.Xborg.Wopr.populations);
	 Poupulation_Available := Boolean'Input(Xborg.Channel);
         if Poupulation_Available then
	    --Text_Io.Put("Population available...");
            Poupulations_Count := Natural(positive'Input(Xborg.Channel));
	    --Text_Io.Put("Population count " & Natural'Image(Poupulations_Count));
            for I in 1..Poupulations_Count loop
               Nation.Manager.Insert(Xborg.Xborg.Wopr.populations, Nation.Manager.Last_Index(Xborg.Xborg.Wopr.Populations)+ 1, nation_Type'Input(Xborg.Channel), 1);
	       --Text_Io.Put("Receive nation " & natural'Image(i));
            end loop;
         end if;
         
	 Xborg.Num := 1;
         Xborg.Items_List := (others => null);

         Clear_canvas(null, Xborg);
         Draw_Wopr_With_Missiles(Xborg);

	 
         
	 
         Xborg.External_Num := 1;
         Xborg.External_Items_List := (others => null);
         if not Weapon.Manager.Is_Empty(Xborg.Xborg.Wopr.enemy) then
            for I in 1..Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Enemy) loop
	       Xborg.Missile_Attributes := Weapon.Manager.Element(Xborg.Xborg.Wopr.Enemy, I).Missile_attributes;
	       declare
	 	  Source_Map_Coordonates : Map_Coordonates_Type := To_Map_Coordonates(Xborg.Missile_Attributes.position);
	       begin		  
	 	     Add_Missile(Xborg,
	 			 Gint(Source_Map_Coordonates.X),
	 			 Gint(Source_Map_Coordonates.Y));			      
	       end;	      
               --delay 0.05;
            end loop;
         end if;


         --  if Is_Empty(Xborg.Xborg.Wopr.Missiles) then
         --     Set_Sensitive(Xborg.Launch_one_Item, False);
         --     Set_Sensitive(Xborg.Launch_All_Item, False);
         --  elsif not Is_Empty(Xborg.Xborg.Wopr.Missiles) then
         --     Set_Sensitive(Xborg.Launch_one_Item, True);
         --     Set_Sensitive(Xborg.Launch_All_Item, True);
         --  end if;
         Set_Sensitive(Xborg.New_Menu_Item, False);
         --Set_Sensitive(Xborg.Create_one_Item, True);
      end if;

      
      --Text_Io.Put_Line("pass 3");
            
      
      
      
      case Xborg.Xborg.Wopr.Defcon is
         when Defcon_5 =>
	    --Text_Io.Put_Line("pass 5");
            Set(Xborg.Defcon_Image, "./data/images/50px-Dc_five_1.svg.png");
	    
            Xborg.Xborg.Operating_Code.Prefix := Random(2);
            Xborg.Xborg.Operating_Code.Radix := Random(3);
            Xborg.Xborg.Operating_Code.suffix := Random(2);
            Set_Label(Xborg.Launch_Code_Label, "Launch Code : " &
                        Xborg.Xborg.Operating_Code.Prefix & " - " &
                        Xborg.Xborg.Operating_Code.Radix & " - " &
                        Xborg.Xborg.Operating_Code.Suffix);
	    Set_Label(Xborg.Destroyed_Missiles_Label, "Destroyed Missiles : " & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Destroyed_Missiles));
	    Set_label(Xborg.Destroyed_Wopr_Label, "Destroyed WOPR  : " & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Destroyed_Wopr));
	    Set_Label(Xborg.Search_Prefix_Label, "Ouest :" & Long_Long_Integer'Image(Xborg.Xborg.Ouest_Population));
	    Set_Label(Xborg.Search_Radix_Label, "Est   :" & Long_Long_Integer'Image(Xborg.Xborg.Est_Population));
	    Xborg.Xborg.Wopr.Defcon := Defcon_4;
	 when Defcon_4 =>
	    --Text_Io.Put_Line("pass 6");
            -- Offensive --
            Set(Xborg.Defcon_Image, "./data/images/53px-Dc_four.png");
            Xborg.Xborg.Offensive := Offensive_Type(Get_Value(Xborg.Strategy));
            Charge_Per_Missile := Float(Get_Value(Xborg.Charge_Max));
            Offensive_Charge := ((Xborg.Xborg.Wopr.Wopr_Attributes.Charge/100.0) *  Float(Xborg.Xborg.Offensive));
	    
            while Offensive_charge >= 0.0 loop
	       if (Offensive_Charge -  Charge_Per_Missile) >= 0.0 then
	       --  declare
	       --  	  Ouest_Target_X : Float;
	       --  	  Est_Target_X  : Float;
	       --  begin
	       --  	     case Xborg.Xborg.Wopr.Wopr_Attributes.Side is
		  
	       --  		when Ouest =>
			   
			   
	       --  		   Xborg.One_Missile.Missile_Attributes.source.X := Random(Position_Gen) * Max_X_Coordonate;
			   
	       --  		   Xborg.One_Missile.Missile_Attributes.source.Y := Random(Position_Gen) * Max_Y_Coordonate;
			   
	       --  		   loop
	       --  		      Ouest_Target_X := Random(Position_Gen);
	       --  		      exit when ouest_Target_X > 0.5;			   
	       --  		   end loop;
	       --  		   Xborg.One_Missile.Missile_Attributes.Target.X := Ouest_Target_X * Max_X_Coordonate;
	       --  		   Xborg.One_Missile.Missile_Attributes.Target.Y := Random(Position_Gen) * Max_Y_Coordonate;
			   
	       --  		when Est =>
			   
	       --  		   Xborg.One_Missile.Missile_Attributes.source.X := Random(Position_Gen) * Max_X_Coordonate;
	       --  		   Xborg.One_Missile.Missile_Attributes.source.Y := Random(Position_Gen) * Max_Y_Coordonate;
			   
	       --  		   loop
	       --  		      est_Target_X := Random(Position_Gen);
	       --  		      exit when est_Target_X < 0.5;			   
	       --  		   end loop;			
	       --  		   Xborg.One_Missile.Missile_Attributes.Target.X := Est_Target_x * Max_X_Coordonate;
	       --  		   Xborg.One_Missile.Missile_Attributes.Target.Y := Random(Position_Gen) * Max_Y_Coordonate;
	       --  	     end case;
	       --  	     Xborg.One_Missile.Missile_Attributes.charge := Charge_Per_missile;
	       --  	  end;
	       
		  		  
	       declare
		     --Ouest_Target_X : Float;
		     --Est_Target_X  : Float;
		     
		  begin
		     case Xborg.Xborg.Wopr.Wopr_Attributes.Side is
			
			when Ouest =>
			   --Text_Io.Put("6 1");
			   if Bool_Random.Random(Bool_Gen) then
			      Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * (-180.0), Random(Position_Gen) * (-90.0));
			   else
			      Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * (-180.0), Random(Position_Gen) * (90.0));
			   end if;
			   --  loop
			   --     --Text_Io.Put("6 2");
			   --     Ouest_Target_X := Random(Position_Gen);
			   --     exit when ouest_Target_X > 0.5;			   
			   --  end loop;
			   --Text_Io.Put("6 3");
			   if Bool_Random.Random(Bool_Gen) then
			      Xborg.One_Missile.Missile_Attributes.Target := To_Gps(Random(Position_Gen) * 180.0, Random(Position_Gen) * (-90.0));
			   else
			      Xborg.One_Missile.Missile_Attributes.Target := To_Gps(Random(Position_Gen) * 180.0, Random(Position_Gen) * (90.0));
			   end if;
			   
			when Est =>
			   --Text_Io.Put("6 4");
			   if Bool_Random.Random(Bool_Gen) then
			      Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * 180.0, Random(Position_Gen) * (-90.0));
			   else
			      Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * 180.0, Random(Position_Gen) * (90.0));
			   end if;
			   --  loop
			   --     --Text_Io.Put("6 5");
			   --     est_Target_X := Random(Position_Gen);
			   --     exit when est_Target_X < 0.5;			   
			   --  end loop;	
			   --Text_Io.Put("6 6");
			   if Bool_Random.Random(Bool_Gen) then
			      Xborg.One_Missile.Missile_Attributes.Target := To_Gps(Random(Position_Gen) * (-180.0), Random(Position_Gen) * (-90.0));
			   else
			      Xborg.One_Missile.Missile_Attributes.Target := To_Gps(Random(Position_Gen) * (-180.0), Random(Position_Gen) * (90.0));
			   end if;
			   
		     end case;
		     --Text_Io.Put("6 7");
		     Xborg.One_Missile.Missile_Attributes.charge := Charge_Per_missile;
		  end;
		  --Text_Io.Put("6 8");
		  Print_Missile(Xborg.One_Missile);
		  --Text_Io.Put("6 9");
		  Command_Type'Output(Xborg.Channel, Missile_Create);
		  --Text_Io.Put("6 10");
		  Missile_Attributes_type'Output(Xborg.Channel, Xborg.One_Missile.Missile_Attributes);
		  --Text_Io.Put("6 11");
		  if Boolean'Input(Xborg.Channel) then
		     --Text_Io.Put("6 12");
		     Xborg.Xborg.Wopr.Wopr_Attributes.Charge :=
		       Xborg.Xborg.Wopr.Wopr_Attributes.Charge - Charge_Per_missile;
		     Offensive_Charge := Offensive_Charge - Charge_Per_Missile;
		     Weapon.Manager.Insert(Xborg.Xborg.Wopr.Weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons) + 1, Xborg.One_Missile, 1);
			
		  end if;
		  --Text_Io.Put("6 13");
	       else
		  exit;
	       end if;
	    end loop;
	    
	    --Text_Io.Put("6 14");
	    Xborg.Xborg.Wopr.Wopr_Attributes.Offensive_Missiles := Natural(Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons));	    	    
	    
	    
	    Set_Label(Xborg.Attack_Missiles_Label, "Offensive Missiles :" & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Offensive_Missiles));
	    
	    --Text_Io.Put("6 15");
	    Gtk.Main.Timeout_Remove(Xborg.Timeout_Id);
	    Xborg.Timeout_Id := Defcon_Timeout.Add(10000, Defcon_Automation'Access, Xborg);
	    Xborg.Xborg.Wopr.Defcon := Defcon_3;
         when Defcon_3 =>
	    --Text_Io.Put_Line("pass 7");
            Set(Xborg.Defcon_Image, "./data/images/53px-Dc_three.png");
            Command_type'Output(Xborg.Channel, Enemy_get);
            Enemy_Available := Boolean'Input(Xborg.Channel);
            if Enemy_Available then
               Enemy_Count := Missile_Vector_Index_Type'Input(Xborg.Channel);
               for I in 1..Enemy_Count loop
                  Xborg.Missile_Attributes := Missile_Attributes_Type'Input(Xborg.Channel);
                  Xborg.One_Missile.Missile_Attributes := Xborg.Missile_Attributes;
		  if Side_Of(Xborg.One_Missile.Missile_Attributes.Position) /= Xborg.Xborg.Wopr.Side then
		     Weapon.Manager.Insert(Xborg.Xborg.Wopr.Enemy, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Enemy)+ 1, Xborg.One_Missile, 1);
		  else
		     Weapon.Manager.Insert(Xborg.Xborg.Wopr.weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.weapons)+ 1, Xborg.One_Missile, 1);
		  end if;
               end loop;
            end if;
	    
	    Xborg.Xborg.Wopr.Wopr_Attributes.Enemy_Missiles := Natural(Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Enemy));
	    	    
	    
	    Set_Label(Xborg.Enemy_missiles_Label, "Enemy Missiles : " & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Enemy_Missiles));
	    
	    
	    

	    
            Xborg.Xborg.Wopr.Defcon := Defcon_2;
         when Defcon_2 =>
	    --Text_Io.Put_Line("pass 8");
            -- Defensive --
	    Xborg.Xborg.Wopr.Wopr_Attributes.Defense_Missiles := 0;
            Set(Xborg.Defcon_Image, "./data/images/53px-Dc_two.png");
	    --Text_Io.Put_Line("Enemy_Count ");
	    Enemy_Count := Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Enemy);
	    --Text_Io.Put_Line("Enemy_Count Ok");
            if Enemy_Count /= 0 then
               Charge_Per_Missile := Float(Get_Value(Xborg.Charge_Max));
               for I in 1..Enemy_Count loop
		  --Text_Io.Put_Line("Enemy_Count " & Integer'Image(I));
                  --while  Last_Index(Xborg.Xborg.Wopr.Enemy) /= 0 loop
		  
                  --Xborg.One_Missile := Weapon.Manager.Element(Xborg.Xborg.Wopr.Enemy, Last_Index(Xborg.Xborg.Wopr.Enemy));
                  Xborg.One_Missile := Weapon.Manager.Element(Xborg.Xborg.Wopr.Enemy, i);
                  --Delete(Xborg.Xborg.Wopr.Enemy, Last_Index(Xborg.Xborg.Wopr.Enemy));		  
		  
		  if Side_Of(Xborg.One_Missile.Missile_Attributes.Position) /= Xborg.Xborg.Wopr.Wopr_Attributes.Side then
			
			Xborg.One_Missile.Missile_Attributes.target := Xborg.One_Missile.Missile_Attributes.Position;		    
		  
		  
		  
			case Xborg.Xborg.Wopr.Wopr_Attributes.Side is
			   when Ouest =>									
			
			      if Bool_Random.Random(Bool_Gen) then
				 Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * (-180.0), Random(Position_Gen) * (-90.0));
			      else
				 Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * (-180.0), Random(Position_Gen) * (90.0));
			      end if;			
			   when Est =>
			      if Bool_Random.Random(Bool_Gen) then
				 Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * 180.0, Random(Position_Gen) * (-90.0));
			      else
				 Xborg.One_Missile.Missile_Attributes.Position := To_Gps(Random(Position_Gen) * 180.0, Random(Position_Gen) * (90.0));
			      end if;			
			      
			end case;
			
			Xborg.One_Missile.Missile_Attributes.Charge := Charge_Per_Missile;
		  --Print_Missile(Xborg.One_Missile.Missile_Attributes);
			Command_Type'Output(Xborg.Channel, Missile_Create);
			Missile_Attributes_type'Output(Xborg.Channel, Xborg.One_Missile.Missile_Attributes);
			if Boolean'Input(Xborg.Channel) then
			   Xborg.Xborg.Wopr.Wopr_Attributes.Charge :=
			     Xborg.Xborg.Wopr.Wopr_Attributes.Charge - Charge_Per_missile;
			   
			   Weapon.Manager.Insert(Xborg.Xborg.Wopr.Weapons, Weapon.Manager.Last_Index(Xborg.Xborg.Wopr.Weapons) + 1, Xborg.One_Missile, 1);
			   
			   Xborg.Xborg.Wopr.Wopr_Attributes.Defense_Missiles := Xborg.Xborg.Wopr.Wopr_Attributes.Defense_Missiles +1;
			   
			end if;
		  end if;
                  exit when Xborg.Xborg.Wopr.Wopr_Attributes.Charge < Charge_Per_missile;
		  
               end loop;
	       
	       
	       
	       

            end if;
	    Set_Label(Xborg.Defense_missiles_Label, "Defensive Missiles : "&Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Defense_Missiles));
	    
	    Gtk.Main.Timeout_Remove(Xborg.Timeout_Id);
	    Xborg.Timeout_Id := Defcon_Timeout.Add(10000, Defcon_Automation'Access, Xborg);
	    
            Xborg.Xborg.Wopr.Defcon := Defcon_1;
         when Defcon_1 =>
	    --Text_Io.Put_Line("pass 9");
	    declare
               task type Launch_Code_Search_type is
               end Launch_Code_Search_type;
               task body Launch_Code_Search_type is
               begin
                  Xborg.Borg := new Borg_Type;
                  Xborg.Borg.Start(Xborg.Xborg.Operating_Code.Prefix);
		  --Set_Label(Xborg.Search_Prefix_Label, Xborg.Xborg.Operating_Code.Prefix);
		  --Queue_Draw(Xborg.Search_Prefix_Label);
                  Xborg.Borg.Start(Xborg.Xborg.Operating_Code.Radix);
		  --Set_Label(Xborg.Search_Radix_Label, Xborg.Xborg.Operating_Code.Radix);
		  --Queue_Draw(Xborg.Search_Radix_Label);
                  Xborg.Borg.Start(Xborg.Xborg.Operating_Code.Suffix);
		  --Set_Label(Xborg.Search_Postfix_label, Xborg.Xborg.Operating_Code.Suffix);
		  --Queue_Draw(Xborg.Search_Postfix_Label);
                  Xborg.Borg.Start("0");
                  Xborg.Borg.Stop;
                  Text_Io.Put_Line("Code Match");
                  Gtk.Dialog.response(Xborg.Code_Dialog, Gtk.Dialog.Gtk_Response_YES);
               end Launch_Code_Search_type;

               Launch_Code_Search : Launch_Code_Search_Type;


               Success : Boolean := False;
               Ok        : Gtk.Widget.Gtk_Widget;
               cancel    : Gtk.Widget.Gtk_Widget;
               Vbox : Gtk_Vbox;
            begin
               Set(Xborg.Defcon_Image, "./data/images/50px-Dc_one_1.svg.png");
               loop
                  Gtk_New(Xborg.Code_Dialog, "Get Launch Code", Xborg.Main_Window, Gtk.dialog.modal);

		  
                  Ok := Gtk.Dialog.Add_Button(Xborg.Code_Dialog, "Ok", Gtk.Dialog.Gtk_Response_Ok);
                  Cancel := Gtk.Dialog.Add_Button(Xborg.Code_Dialog, "Cancel", Gtk.Dialog.Gtk_Response_Cancel);

                  Vbox := Get_Vbox(Xborg.Code_Dialog);
                  Set_Position(Xborg.Code_Dialog, Win_Pos_Center);
                  Gtk_New(Xborg.Code_Entry);
                  Pack_Start(Vbox, Xborg.Code_Entry);

                  Show_All(Xborg.Code_Dialog);
                  case Gtk.Dialog.Run(Xborg.Code_Dialog) is
                     when Gtk.Dialog.Gtk_Response_Ok =>
                        if Get_Text(Xborg.Code_Entry) =
                          Xborg.Xborg.Operating_Code.Prefix &
                          Xborg.Xborg.Operating_Code.Radix &
                          Xborg.Xborg.Operating_Code.Suffix then
                           Success := True;
                           abort Launch_Code_Search;
                           Xborg.Borg.Stop;
                           Gtk.Dialog.Destroy(Xborg.Code_Dialog);
                           exit;
                        end if;
                        Gtk.Dialog.Destroy(Xborg.Code_Dialog);
                     when Gtk.Dialog.Gtk_Response_Cancel =>
                        abort Launch_Code_Search;
                        Xborg.Borg.Stop;
                        Gtk.Dialog.Destroy(Xborg.Code_Dialog);
                        exit;
                     when Gtk.Dialog.Gtk_Response_Yes =>
                        Gtk.Dialog.Destroy(Xborg.Code_Dialog);
                        exit;
                     when others =>
                         abort Launch_Code_Search;
                         Xborg.Borg.Stop;
                         Gtk.Dialog.Destroy(Xborg.Code_Dialog);
                        exit;
                  end case;
               end loop;

               if not Success then
                  Launch_All(null, Xborg);
                  Weapon.Manager.Clear(Xborg.Xborg.Wopr.weapons);
               end if;
            end;
            
	    Xborg.Xborg.Wopr.Wopr_Attributes.Enemy_Missiles := 0;
	    Set_Label(Xborg.Enemy_missiles_Label, "Enemy Missiles : " & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Enemy_Missiles));
	    Xborg.Xborg.Wopr.Wopr_Attributes.Offensive_Missiles := 0;
	    Set_Label(Xborg.Attack_Missiles_Label, "Offensive Missiles : " & Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Offensive_Missiles));	       
	    Xborg.Xborg.Wopr.Wopr_Attributes.Defense_Missiles := 0;
	    Set_Label(Xborg.Defense_missiles_Label, "Defense Missiles : "& Natural'Image(Xborg.Xborg.Wopr.Wopr_Attributes.Defense_Missiles));
	    --Set_Label(Xborg.Search_Prefix_Label, "");

	    --Set_Label(Xborg.Search_Radix_Label, "");
	    
	    --Set_Label(Xborg.Search_Postfix_label, "");
	    
	    

	    --Text_Io.Put_Line("pass 12");
            Weapon.Manager.Clear(Xborg.Xborg.Wopr.Enemy);
	    
	    Gtk.Main.Timeout_Remove(Xborg.Timeout_Id);
	    Xborg.Timeout_Id := Defcon_Timeout.Add(10000, Defcon_Automation'Access, Xborg);
	    
            Xborg.Xborg.Wopr.Defcon := Defcon_5;
      end case;
      

      return True;
   end Defcon_Automation;

   procedure Initialize(Xborg : out Xborg_Application;
                        Channel  : in Stream_Access;
                        Wopr  : in out Joshua.Joshua_Type) is
      
      --  Max_Geometry                   : Gdk_Geometry := (810, 800, 810, 800,
      --                                                    810, 800, 810, 800, 1.0, 1.0,
      --                                                    Gravity_Center);

      --  Min_Geometry                   : Gdk_Geometry := (810, 800, 810, 800,
      --                                                    810, 800, 810, 800, 1.0, 1.0,
      --                                                    Gravity_Center);

      Update_Id : Gtk.Main.Timeout_Handler_Id;
      
      
      Observation_Id : Gtk.Main.Timeout_Handler_Id;
      

      
   begin
      
      Xborg := new Xborg_context;
      Xborg.Xborg.Wopr := Wopr;
      Xborg.Channel := Channel;
      Gtk_New(Xborg.Main_Window, Window_Toplevel);

      Set_Title(Xborg.Main_Window, "Xborg 2014");

      Set_Position(Xborg.Main_Window, Win_Pos_Center);      
      Set_Default_Size(Xborg.Main_Window, 1280, 1024);
      Set_Resizable(Xborg.Main_Window, False);
      --Set_Geometry_Hints(Xborg.Main_Window, null,
      --                   Max_Geometry, Gdk_Hint_Max_size);
      --Set_Geometry_Hints(Xborg.Main_Window, null,
      --                   Min_Geometry, Gdk_Hint_Min_size);



      Gtk_New_Vbox(Xborg.Main_Vbox);
      Gtk_New(Xborg.Menu_Bar);
      Gtk_New(Xborg.File_Menu);
      Gtk_New(Xborg.File_Menu_Item, "Game");
      Gtk_New(Xborg.New_Menu_Item, "New");
      Gtk_New(Xborg.Exit_Menu_Item, "Quit");

      Set_Submenu(Xborg.File_Menu_Item, Xborg.File_Menu);
      Append(Xborg.File_Menu, Xborg.New_Menu_Item);
      Set_Sensitive(Xborg.New_Menu_Item, False);
      Append(Xborg.File_Menu, Xborg.Exit_Menu_Item);
      Append(Xborg.Menu_Bar, Xborg.File_Menu_Item);

      --  Gtk_New(Xborg.Missiles_Menu);
      --  Gtk_New(Xborg.Missiles_Menu_Item, "Missiles");
      --  Gtk_New(Xborg.Launch_all_Item, "Launch All");
      --  Gtk_New(Xborg.Launch_One_Item, "Launch One");
      --  Gtk_New(Xborg.Create_One_Item, "Create One");
      --  Set_Submenu(Xborg.Missiles_Menu_Item, Xborg.Missiles_Menu);
      --  Append(Xborg.Missiles_Menu, Xborg.Launch_All_Item);
      --  Set_Sensitive(Xborg.Launch_All_Item, False);
      --  Append(Xborg.Missiles_Menu, Xborg.Launch_One_Item);
      --  Set_Sensitive(Xborg.Launch_One_Item, False);
      --  Append(Xborg.Missiles_Menu, Xborg.Create_One_Item);
      --  Set_Sensitive(Xborg.Create_One_Item, False);
      --  Append(Xborg.Menu_Bar, Xborg.Missiles_Menu_item);

      Gtk_New_Hbox(Xborg.Tools_Hbox);
      Pack_start(Xborg.Tools_Hbox, Xborg.Menu_bar, False, false);
      Gtk_New(Xborg.Charge_Max, 0.01, 0.5, 0.01);
      Set_Value(Xborg.Charge_Max, 0.25);
      Gtk_New(Xborg.Charge_Max_Label, "Charge Max : ");
      Gtk_New(Xborg.Strategy, 1.0, 99.0, 1.0);
      Set_Value(Xborg.Strategy, 50.0);
      Gtk_New(Xborg.Strategy_Label, "Offensive Strategy : ");
      Pack_start(Xborg.Tools_Hbox, Xborg.Strategy_Label, False, False, 5);
      Pack_start(Xborg.Tools_Hbox, Xborg.strategy, False, false);
      Pack_start(Xborg.Tools_Hbox, Xborg.Charge_Max_Label, False, False,5);
      Pack_start(Xborg.Tools_Hbox, Xborg.Charge_Max, false, true, 5);
      Gtk_New(Xborg.Launch_Code_Label, "Launch Code : ");
      Pack_Start(Xborg.Tools_Hbox, Xborg.Launch_Code_Label, True, True );
      Pack_start(Xborg.Main_Vbox, Xborg.Tools_hbox, False, false);
      Gtk_New(Xborg.Main_Frame);


      Gtkada.Canvas.initialize(Xborg.Canvas, True);
      Set_Size_Request(Xborg.Canvas, Gint(width)+10, Gint(heigh)+10);
      
      Gtk_New(Xborg.Points_Frame, "Points");

      Gtk_New_Vbox(Xborg.Points_Vbox);
      Gtk_New(Xborg.Destroyed_Wopr_Label, "Destroyed WOPR  : ");
      Gtk_New(Xborg.Earned_missiles_Label, "Earned Missiles : ");
      Gtk_New(Xborg.Destroyed_Missiles_Label, "Destroyed Missiles : ");
      Pack_Start(Xborg.points_Vbox, Xborg.Destroyed_Missiles_Label, False, True);      
      Pack_Start(Xborg.Points_Vbox, Xborg.Destroyed_Wopr_Label, False, true);
      Pack_Start(Xborg.Points_Vbox, Xborg.Earned_Missiles_Label, False, true);
      Add(Xborg.Points_Frame, Xborg.Points_Vbox);
      
      --Gtk_New(Xborg.Search_Frame, "Searching Code");
      --Gtk_New_Vbox(Xborg.search_Vbox);
      --Add(Xborg.Search_Frame, Xborg.Search_Vbox);
      

      --Gtk_New(Xborg.Search_Prefix_Label, "");
      --Pack_Start(Xborg.Search_Vbox, Xborg.Search_Prefix_Label, False, True);
      --Gtk_New(Xborg.Search_Radix_Label, "");
      --Pack_Start(Xborg.Search_Vbox, Xborg.Search_radix_Label, False, True);
      --Gtk_New(Xborg.Search_Postfix_Label, "");
      --Pack_Start(Xborg.Search_Vbox, Xborg.Search_Postfix_Label, False, True);
      
      
      Gtk_New(Xborg.Search_Frame, "Populations");
      Gtk_New_Vbox(Xborg.search_Vbox);
      Add(Xborg.Search_Frame, Xborg.Search_Vbox);
      

      Gtk_New(Xborg.Search_Prefix_Label, "Ouest :");
      Pack_Start(Xborg.Search_Vbox, Xborg.Search_Prefix_Label, False, True);
      Gtk_New(Xborg.Search_Radix_Label, "Est   :");
      Pack_Start(Xborg.Search_Vbox, Xborg.Search_radix_Label, False, True);
      --Gtk_New(Xborg.Search_Postfix_Label, "");
      --Pack_Start(Xborg.Search_Vbox, Xborg.Search_Postfix_Label, False, True);
      
      
	  
      Gtk_New(Xborg.Log_Frame, "Log");
      Gtk_New(Xborg.Defcon_Frame, "Defcon");
      Gtk_New_Hbox(Xborg.Bot_Hbox);
      Gtk_New_vbox(Xborg.Left_Vbox);
      
      Pack_Start(Xborg.Left_Vbox, Xborg.Defcon_Frame, False, True, 5);

      Gtk_New_vbox(Xborg.Log_Vbox);      
      Gtk_New(Xborg.Elapsed_Time_Label, "Elapsed time : ");
      Pack_Start(Xborg.Log_Vbox, Xborg.Elapsed_Time_Label, False, True);
      
      Gtk_New(Xborg.Attack_Missiles_Label, "Offensive Missiles :");
      Pack_Start(Xborg.Log_Vbox, Xborg.Attack_Missiles_Label, False, True);
      
      Gtk_New(Xborg.Enemy_missiles_Label, "Enemy Missiles : ");
      Pack_Start(Xborg.Log_Vbox, Xborg.Enemy_missiles_Label, False, True);
      
      Gtk_New(Xborg.Defense_missiles_Label, "Defense Missiles : ");
      Pack_Start(Xborg.Log_Vbox, Xborg.Defense_Missiles_Label, False, True);      
      
      Gtk_New(Xborg.Stats_Frame, "Stats");      
      Gtk_New_Vbox(Xborg.Stats_Vbox);
      Gtk_New(Xborg.Destroyed_Missiles_Per_Houre, "Frequency : ");
      Gtk_New(Xborg.Elapsed_Houre_Label, "Elapsed : " & Formatting.Image(0.0));
      Pack_Start(Xborg.Stats_Vbox, Xborg.Destroyed_Missiles_Per_Houre, False, False);
      Pack_Start(Xborg.Stats_Vbox, Xborg.Elapsed_Houre_Label, False, false);
      
      Add(Xborg.Stats_frame, Xborg.stats_Vbox);      
      Add(Xborg.Log_Frame, Xborg.Log_Vbox);
      Pack_Start(Xborg.Bot_Hbox, Xborg.Stats_Frame, False, True, 5);
      Pack_Start(Xborg.left_Vbox, Xborg.Search_Frame, False, True, 5);
      Pack_Start(Xborg.Bot_Hbox, Xborg.Log_Frame, True, True, 5);
      Pack_Start(Xborg.Bot_Hbox, Xborg.Points_Frame, True, True, 5);
      
      Gtk_New_Hbox(Xborg.Main_Hbox);
      
      Pack_start(Xborg.Main_Hbox, Xborg.Left_vbox);
      Pack_start(Xborg.Main_Hbox, Xborg.Canvas);
      Pack_start(Xborg.Main_Vbox, Xborg.Main_Hbox);
      Pack_Start(Xborg.Main_Vbox, Xborg.Bot_Hbox);
      
      Add(Xborg.Main_frame, Xborg.Main_vbox);
      Add(Xborg.Main_Window, Xborg.Main_frame);

      for J in Color_Names'Range loop
         Colors (J) := Parse (Color_Names (J).all);
         Alloc (Gtk.Widget.Get_Default_Colormap, Colors (J));
      end loop;
      Realize (Xborg.Canvas);
      Set_Background(null, Xborg);
      Gdk_New (Green_Gc, Get_Window (Xborg.Canvas));
      Layout := Create_Pango_Layout (Xborg.Main_frame);

      Gtk_New(Xborg.Defcon_Image);
      Set(Xborg.Defcon_Image, "./data/images/50px-Dc_five_1.svg.png");
      Add(Xborg.Defcon_Frame, Xborg.Defcon_Image);

      
      

      Main_Return_Handlers.Connect
        (Xborg.Main_Window, "delete_event", Main_Return_Handlers.To_Marshaller (Main_destroy'access), Xborg);

      Main_Handlers.Connect
        (Xborg.Exit_Menu_Item, "activate", Main_Handlers.To_Marshaller (Main_Quit'access), Xborg);

      Main_handlers.Connect
        (Xborg.New_Menu_item, "activate",  Main_Handlers.To_Marshaller (Get_Wopr_Attributes'Access), Xborg);

      --  Main_handlers.Connect
      --    (Xborg.Create_One_item, "activate",  Main_Handlers.To_Marshaller (Get_Missile_attributes'Access), Xborg);

      --  Main_handlers.Connect
      --    (Xborg.Launch_One_item, "activate",  Main_Handlers.To_Marshaller (Get_one'Access), Xborg);

      --  Main_handlers.Connect
      --    (Xborg.Launch_All_item, "activate",  Main_Handlers.To_Marshaller (Launch_All'Access), Xborg);



      Show_All(Xborg.Main_window);
      
      Get_Wopr_Attributes(null, Xborg);
      
      Update_Id := Defcon_Timeout.Add(1000, Update'Access, Xborg);
      observation_Id := Defcon_Timeout.Add(1000, observation'Access, Xborg);
   end Initialize;

end Warborg.Xborg;
