
-- Warborg is Wargames playing on network
-- Copyright (C) 2011 Manuel De Girardi

-- This Program is Free Software: You Can Redistribute It and/or Modify
-- It Under The Terms of The GNU General Public License As Published By
-- The Free Software Foundation, Either Version 3 of The License, or
-- (at Your Option) Any Later Version.

-- This Program is Distributed in The Hope That It Will Be Useful,
-- But WITHOUT ANY WARRANTY; Without Even The Implied Warranty of
-- MERCHANTABILITY or FITNESS for A PARTICULAR PURPOSE.  See The
-- GNU General Public License for More Details.

-- You Should Have Received A Copy of The GNU General Public License
-- Along with This Program.  if not, See <Http://Www.Gnu.Org/Licenses/>.

with Warborg.Common;
use Warborg.Common, Warborg.Common.My_Map;
with Warborg.World;
use Warborg.World;

with Warborg.User.Manager;
with Warborg.User.Util;
use Warborg.User;
with Warborg.Nation.Manager;
use Warborg.Nation.Manager;
use Warborg.Nation;
with Warborg.Joshua;
with Warborg.Weapon;




with Gnat.SHA1;
use Gnat;

with Ada.Calendar;                     use Ada.Calendar;
with Ada.Finalization;                 use Ada.Finalization;
with Gnat.Semaphores, Gnat.Sockets;    use Gnat.Semaphores, Gnat.Sockets;
with Gtk.Main;
with Gtk.Window;                       use Gtk.Window;
with Gtk.Dialog;                        use Gtk.Dialog;
with Gtk.Enums;                        use Gtk.Enums;
with Gtkada.Canvas;                     use Gtkada.Canvas;
with Gdk.Drawable;                      use Gdk.Drawable;
with Gdk.Pixbuf;                        use Gdk.Pixbuf;
with Gdk.GC;                            use Gdk.GC;
with Gdk.Rectangle;                     use Gdk.Rectangle;
with Gdk.Color;                         use Gdk.Color;
with Glib;                               use Glib;
with Gtk.Spin_Button;                   use Gtk.Spin_Button;
with Gtk.Box;                           use Gtk.Box;
with Gtk.Label;                         use Gtk.Label;
with Gtk.Frame;                         use Gtk.Frame;
with Gtk.Menu_Bar;                      use Gtk.Menu_Bar;
with Gtk.Menu;                          use Gtk.Menu;
with Gtk.Menu_Item;                     use Gtk.Menu_Item;
with Gtk.Image;                         use Gtk.Image;
with Gtk.Gentry;                         use Gtk.Gentry;

package Warborg.Xborg is

   use Joshua;

   subtype Prefix_Type is String(1..2);
   subtype Radix_Type is String(1..3);
   subtype suffix_Type is String(1..2);
   type Code_Type is
      record
         Prefix : Prefix_Type;
         Radix  : Radix_Type;
         Suffix : Suffix_Type;
      end record;


   subtype Offensive_Type is Natural range 0..99;

   type Xborg_Type is
      record
         Auto            : Boolean := False;
         Lock            : Binary_Semaphore(True, Default_Ceiling);                  
         Operating_Code  : Code_Type := (Prefix => (others => Character'Val(32)),
                                         radix => (others => Character'Val(32)),
                                         suffix => (others => Character'Val(32)));
         Start_Date      : Time;
         Time_Remainning : Duration := 0.0;
         wopr            : Joshua.Joshua_Type;
         Offensive       : Offensive_Type := 50;                  
	 
	 Ouest_Population : Long_Long_Integer := 0;
	 Est_Population    : Long_Long_Integer := 0;	 
      end record;


   type Image_Canvas_Record is new Interactive_Canvas_Record with record
      Background : Gdk_Pixbuf;
      Draw_Grid  : Boolean := False;
      Grid_GC    : Gdk_GC;
   end record;

   type Image_Canvas is access all Image_Canvas_Record'Class;

   procedure Draw_Background
     (Canvas        : access Image_Canvas_Record;
      Screen_Rect   : Gdk.Rectangle.Gdk_Rectangle);

   type Display_Item_Record is new buffered_Item_Record with record
      Canvas : Interactive_Canvas;
      Color  : Gdk.Color.Gdk_Color;
      W, H   : Gint;
      Num    : Positive;
   end record;
   type Display_Item is access all Display_Item_Record'Class;


   Max_Colors : constant := 20;
   type Color_Type is range 1 .. Max_Colors;
   Color_Names : constant array (Color_Type) of String_Access :=
     (new String'("forest green"),
      new String'("red"),
      new String'("blue"),
      new String'("yellow"),
      new String'("peach puff"),
      new String'("azure"),
      new String'("seashell"),
      new String'("lavender"),
      new String'("grey"),
      new String'("turquoise"),
      new String'("khaki"),
      new String'("tan"),
      new String'("orange red"),
      new String'("MediumPurple"),
      new String'("ivory1"),
      new String'("DeepSkyBlue1"),
      new String'("burlywood1"),
      new String'("wheat1"),
      new String'("orange1"),
      new String'("pink"));

   Colors : array (Color_Type) of Gdk_Color;


   type Wopr_Attributes_Window is
      record
         Dialog : Gtk_dialog;

         Degr_Lat : Gtk_Spin_Button;
         Minu_Lat : Gtk_Spin_Button;
         Seco_Lat : Gtk_Spin_Button;
         Degr_Lon : Gtk_Spin_Button;
         Minu_Lon : Gtk_Spin_Button;
         Seco_Lon : Gtk_Spin_Button;
         Lat_Hbox : Gtk_Hbox;
         Lon_Hbox : Gtk_Hbox;

         Lat_Label: Gtk_Label;
         Lon_Label : Gtk_Label;

         Main_Vbox : Gtk_Vbox;
      end record;

   type Missile_Attributes_Window is
      record
         Dialog          : Gtk_Dialog;
         Source_Label    : Gtk_Label;
         Source_Degr_Lat : Gtk_Spin_Button;
         Source_Minu_Lat : Gtk_Spin_Button;
         Source_Seco_Lat : Gtk_Spin_Button;
         Source_Degr_Lon : Gtk_Spin_Button;
         Source_Minu_Lon : Gtk_Spin_Button;
         Source_Seco_Lon : Gtk_Spin_Button;
         Source_Lat_Hbox : Gtk_Hbox;
         Source_Lon_Hbox : Gtk_Hbox;
         Target_Label    : Gtk_Label;
         Target_Degr_Lat : Gtk_Spin_Button;
         Target_Minu_Lat : Gtk_Spin_Button;
         Target_Seco_Lat : Gtk_Spin_Button;
         Target_Degr_Lon : Gtk_Spin_Button;
         Target_Minu_Lon : Gtk_Spin_Button;
         Target_Seco_Lon : Gtk_Spin_Button;
         Target_Lat_Hbox : Gtk_Hbox;
         Target_Lon_Hbox : Gtk_Hbox;
         Charge          : Gtk_Spin_Button;
         Charge_Label    : Gtk_Label;
         Charge_Hbox     : Gtk_Hbox;
         Lat_Label: Gtk_Label;
         Lon_Label : Gtk_Label;

         Main_Vbox : Gtk_Vbox;
      end record;

   type Missil_Unit_Config is
      record
         Source_Degr_Lat : Gtk_Spin_Button;
         Source_Minu_Lat : Gtk_Spin_Button;
         Source_Seco_Lat : Gtk_Spin_Button;
         Source_Degr_Lon : Gtk_Spin_Button;
         Source_Minu_Lon : Gtk_Spin_Button;
         Source_Seco_Lon : Gtk_Spin_Button;
         Target_Degr_Lat : Gtk_Spin_Button;
         Target_Minu_Lat : Gtk_Spin_Button;
         Target_Seco_Lat : Gtk_Spin_Button;
         Target_Degr_Lon : Gtk_Spin_Button;
         Target_Minu_Lon : Gtk_Spin_Button;
         Target_Seco_Lon : Gtk_Spin_Button;
         Charge          : Gtk_Spin_Button;

      end record;

   --type Missil_Unit_Access is access all Missil_Unit_Config;

   type Item_Table_Type is array (Positive range <>) of aliased Canvas_Item;
   
      task type Borg_Type is
      entry Start(Code : in String);
      entry Stop;
   end Borg_Type;

   type Borg_Access is access all Borg_Type;

   
   type Xborg_context is new Limited_Controlled with
      record
         Channel                    : Stream_Access;
         Logname                    : String_Access;
         Success                    : Boolean := False;
         Xborg                      : Xborg_Type;         	 
	 Borg                       : Borg_Access;
         Wopr_Window                : Wopr_Attributes_Window;
         Main_Window                             : Gtk_Window;
         Main_Frame                 : Gtk_Frame;
         Main_Vbox                  : Gtk_Vbox;
         Main_Hbox                  : Gtk_Hbox;
	 Left_Vbox                  : Gtk_VBox;
         Bot_HBox                   : Gtk_Vbox;
         Canvas                                  : Image_Canvas := new Image_Canvas_Record;
         Points_Frame               : Gtk_Frame;
         Points_Vbox                : Gtk_Vbox;
         Earned_Missiles_Label      : Gtk_Label;
         Destroyed_Wopr_Label       : Gtk_Label;

         Log_Frame                  : Gtk_Frame;
         Log_Vbox                   : Gtk_Vbox;
         Elapsed_Time_Label         : Gtk_Label;
         Launch_Code_Label          : Gtk_Label;

	 
	 
	 Attack_Missiles_Label      : Gtk_Label; 
	 Enemy_Missiles_label       : Gtk_Label; 
	 Defense_Missiles_label     : Gtk_Label; 
	 Destroyed_Missiles_label   : Gtk_Label; 
	 
	 Search_Frame               : Gtk_Frame;
	 Search_Vbox                : Gtk_Box;
	 Search_Prefix_Label        : Gtk_Label;
	 Search_radix_Label         : Gtk_Label;
	 Search_postfix_Label       : Gtk_Label;
	 
	 
	 Stats_Frame               : Gtk_Frame;
	 Stats_Vbox                : Gtk_Box;
	 Destroyed_Missiles_Per_Houre  : Gtk_Label;
	 End_Time_Of_Observation   : Time := Clock + 3600.0;
	 Total_Destroyed_Per_Houre  : float := 0.0;
	 Elapsed_Houre             : Duration := 3600.0;
	 Elapsed_Houre_label        : Gtk_Label;
	 
         Defcon_Frame               : Gtk_Frame;
         Defcon_Image               : Gtk_Image;
         Strategy_Label             : Gtk_Label;
         Strategy                   : Gtk_Spin_Button;
         Charge_Max_Label           : Gtk_Label;
         Charge_Max                 : Gtk_Spin_Button;
         Tools_Hbox                 : Gtk_Hbox;
         Menu_bar                   : Gtk_Menu_Bar;
         File_Menu                  : Gtk_Menu;
         File_Menu_Item             : Gtk_Menu_Item;
         New_Menu_Item              : Gtk_Menu_Item;
         Exit_Menu_Item             : Gtk_Menu_Item;

         --Missiles_Menu              : Gtk_Menu;
         --Missiles_Menu_item         : Gtk_Menu_Item;
         --Launch_All_item            : Gtk_Menu_Item;
         --Launch_One_item            : Gtk_Menu_Item;
         --Create_One_Item            : Gtk_Menu_Item;

         Missile                    : Missile_Attributes_Window;
	 




         Items_List                 : Item_Table_type(1..50_000);
         Num                        : Positive := 1;

         Missiles_Count             : Extended_Index := 0;
         Missile_Attributes         : Weapon.Missile_Attributes_Type;
         One_Missile                : Weapon.Missile_Type;
         One_Missile_Launch         : Gtk_Spin_Button;
         Launch_Window              : Gtk_Dialog;
         Timeout_Id                 : Gtk.Main.Timeout_Handler_Id;
	 
         Code_Dialog                : Gtk_Dialog;
         Code_Entry                 : Gtk_Gentry;
	 
         External_Items_List        : Item_Table_type(1..150_000);
         External_Num               : Positive := 1;
      end record;
   
   type Xborg_Application is access all Xborg_context;

   procedure Initialize(Xborg    : out Xborg_Application;
                        Channel  : in Stream_Access;
                        Wopr     : in out Joshua.Joshua_type);
   


end Warborg.Xborg;
